@@ -1,7 +1,7 @@
 export CVS_RSH=ssh
-export VN=0.29
+export VN=0.29.1
 export VER=tork-$VN
-export TAG=v0_29
+export TAG=v0_29_1
 cd ..
 TOPDIR=$PWD
 cd TorkReleases
