@@ -1,5 +1,5 @@
 /***************************************************************************
-** $Id: configdialog.cpp,v 1.45 2007/12/23 14:39:48 hoganrobert Exp $
+** $Id: configdialog.cpp,v 1.49 2008/03/25 20:14:02 hoganrobert Exp $
 *   Copyright (C) 2006 by Robert Hogan                                    *
 *   robert@roberthogan.net                                                *
  *                                                                         *
@@ -83,7 +83,7 @@ email                : markey@web.de
 //////////////////////////////////////////////////////////////////////////////////////////
 
 TorkConfigDialog::TorkConfigDialog( QWidget *parent, const char* name, KConfigSkeleton *config )
-        : KConfigDialog( parent, name, config )
+        : KConfigDialog( parent, name, config ),m_parent(parent)
 {
     setWFlags( WDestructiveClose );
 
@@ -93,7 +93,7 @@ TorkConfigDialog::TorkConfigDialog( QWidget *parent, const char* name, KConfigSk
         m_torservers = new TorServers( 0, "My Network View" );
 	    m_maxmin = new MaxMin( 0, "My Server Bandwidth" );
 	    m_running = new Running( 0, "My Tor Client" );
-        m_myserver = new MyServer( 0, "My Tor Server" );
+        m_myserver = new MyServer( this, "My Tor Server" );
         m_myhidden = new MyHidden( 0, "My Hidden" );
         m_quickconfig = new QuickConfig( 0, "Quick Configure" );
         m_konq = new Konq( 0, "Konqueror" );
@@ -106,17 +106,30 @@ TorkConfigDialog::TorkConfigDialog( QWidget *parent, const char* name, KConfigSk
     addPage( m_running, i18n( "My Tor Client" ), "launch", i18n( "Configure My Client" ) );
 #endif
     addPage( m_firewallsproxies, i18n( "Firewall/Censor Evasion" ), "proxy", i18n( "Configure Firewalls Proxies" ) );
+
+    if (TorkConfig::advancedMode()){
 #ifndef LIVECD
 #ifdef USE_FAILSAFE
-    addPage( m_filtertable, i18n( "FailSafe" ), "gv", i18n( "Configure FailSafe Settings" ) );
+      addPage( m_filtertable, i18n( "FailSafe" ), "gv", i18n( "Configure FailSafe Settings" ) );
 #endif
 #endif
-    addPage( m_usability,   i18n( "Usability" ), "idea", i18n( "Configure Usability" ) );
-    addPage( m_torservers, i18n( "My Network View" ), "network", i18n( "Configure My Network View" ) );
+      addPage( m_usability,   i18n( "Usability" ), "idea", i18n( "Configure Usability" ) );
+    }
+    if ((!TorkConfig::excludeNodes().isEmpty()) ||
+        (!TorkConfig::exitNodes().isEmpty())){
+        addPage( m_torservers, i18n( "My Network View" ), "network", 
+                 i18n( "Configure My Network View" ) );
+    }
 #ifndef LIVECD
-    addPage( m_myserver, i18n( "My Tor Server" ), "system", i18n( "Configure My Server" ) );
-    addPage( m_maxmin, i18n( "My Server Bandwidth" ), "filter", i18n( "Configure My Bandwidth" ) );
-    addPage( m_myhidden, i18n( "My Hidden Services" ), "tork_agent", i18n( "Configure My Hidden Services" ) );
+    if (!TorkConfig::clientOnly()){
+        addPage( m_myserver, i18n( "My Tor Server" ), "system", i18n( "Configure My Server" ) );
+        addPage( m_maxmin, i18n( "My Server Bandwidth" ), "filter", i18n( "Configure My Bandwidth" ) );
+    }
+
+    if (!TorkConfig::hiddenServices().isEmpty())
+        addPage( m_myhidden, i18n( "My Hidden Services" ), "tork_agent", 
+                         i18n( "Configure My Hidden Services" ) );
+    
 #endif
     QObjectList *list = queryList( "QLabel", "infoPixmap" );
     for( QObject *label = list->first(); label; label = list->next() )
@@ -147,8 +160,18 @@ TorkConfigDialog::TorkConfigDialog( QWidget *parent, const char* name, KConfigSk
     connect( m_quickconfig, SIGNAL( configChanged(const int &) ), this, SLOT( showHidePages(const int &) ) );
     //connect( m_usability, SIGNAL(settingsChanged()), SLOT(updateButtons()) ); 
 
-    connect( this, SIGNAL(settingsChanged()), parent, SLOT(applySettingsToRunningTor()) );
-    connect( this, SIGNAL(updateServerButton( )), parent, SLOT(updateServerButton( )) );
+    // Always apply settings when OK or Apply is pressed.
+    //connect( this, SIGNAL(settingsChanged()), this,
+    //               SLOT(updateCustomSettings()) );
+    connect( this, SIGNAL(okClicked()), this,
+                   SLOT(updateCustomSettings()) );
+    connect( this, SIGNAL(applyClicked()), this,
+                   SLOT(updateCustomSettings()) );
+    connect( this, SIGNAL(updateServerButton( )), parent,
+                    SLOT(updateServerButton( )) );
+
+    connect( m_firewallsproxies, SIGNAL(uncensorSelected( bool )), parent,
+                                 SLOT(toggleServerButton( bool )) );
 
     if (TorkConfig::currentTorVersion().left(3) == "0.1"){
         m_filtertable->setEnabled(false);
@@ -226,9 +249,14 @@ void TorkConfigDialog::enableApply()
     enableButtonApply(true);
 }
 
-void TorkConfigDialog::updateSettings()
+void TorkConfigDialog::updateCustomSettings()
 {
 
+    kdDebug() << "applied config" << endl;
+
+    //When settings are changed in the config dialog, this is called
+    //to update all possibly changed values and apply them to Tor.
+
     //QListViews need to be handled outside of kconfigxt
     TorkConfig::setQuickConfigure( m_quickconfig->QuickConfigure->currentItem() );
 
@@ -436,19 +464,19 @@ void TorkConfigDialog::updateSettings()
         TorkConfig::setBridge( QStringList() );
 
     //Hidden Services
-    if ( m_myhidden->servicesList->childCount() > 0 ) {
-        QStringList v_hiddenServices;
-        QListViewItemIterator it( m_myhidden->servicesList );
-        while ( it.current() ) {
-            QString s_hiddenServices = it.current()->text(0) + "\n" + it.current()->text(1)
-                                        + "\n" + it.current()->text(2) + "\n" + it.current()->text(3)
-                                        + "\n" + it.current()->text(4)+ "\n" + it.current()->text(5);
-            v_hiddenServices.append(s_hiddenServices);
-	        ++it;
-        }
-        TorkConfig::setHiddenServices( v_hiddenServices );
-    }else
-        TorkConfig::setHiddenServices( "" );
+//     if ( m_myhidden->servicesList->childCount() > 0 ) {
+//         QStringList v_hiddenServices;
+//         QListViewItemIterator it( m_myhidden->servicesList );
+//         while ( it.current() ) {
+//             QString s_hiddenServices = it.current()->text(0) + "\n" + it.current()->text(1)
+//                                         + "\n" + it.current()->text(2) + "\n" + it.current()->text(3)
+//                                         + "\n" + it.current()->text(4)+ "\n" + it.current()->text(5);
+//             v_hiddenServices.append(s_hiddenServices);
+// 	        ++it;
+//         }
+//         TorkConfig::setHiddenServices( v_hiddenServices );
+//     }else
+//         TorkConfig::setHiddenServices( "" );
 
     //Bandwidth Slots
     if ( m_maxmin->timedBandwidthList->childCount() > 0 ) {
@@ -474,9 +502,10 @@ void TorkConfigDialog::updateSettings()
     TorkConfig::writeConfig();
 
 
-    //If you're going to disable this again, explain why here. You have a crap memory.
-    emit settingsChanged(); 
-
+    //If you're going to disable this again, explain why here. You have a crap
+    //memory.
+    //emit settingsChanged(); 
+    static_cast<tork*>(m_parent)->applySettingsToRunningTor();
 }
 
 
