@@ -1,5 +1,5 @@
 /***************************************************************************
-** $Id: crypto.cpp,v 1.9 2007/12/21 19:14:26 hoganrobert Exp $
+** $Id: crypto.cpp,v 1.10 2008/02/12 21:16:15 hoganrobert Exp $
  *   Copyright (C) 2006 by Robert Hogan                                    *
  *   robert@roberthogan.net                                                *
  *                                                                         *
@@ -36,10 +36,16 @@
  * Copyright (c) 2001-2004, Roger Dingledine.
  * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson. */
 /* See Tor LICENSE for licensing information */
-/* $Id: crypto.cpp,v 1.9 2007/12/21 19:14:26 hoganrobert Exp $ */
-#include <string.h> 
+/* $Id: crypto.cpp,v 1.10 2008/02/12 21:16:15 hoganrobert Exp $ */
+#include <string.h>
+#include <kdebug.h>  
 #include "crypto.h"
+#include "../config.h"
 
+#ifndef USE_OPENSSL
+#include <gnutls/gnutls.h>
+#include <gcrypt.h>
+#else
 #include <openssl/err.h>
 #include <openssl/rsa.h>
 #include <openssl/pem.h>
@@ -51,6 +57,7 @@
 #include <openssl/rsa.h>
 #include <openssl/dh.h>
 #include <openssl/conf.h>
+#endif
 
 #include <stdlib.h>
 #include <assert.h>
@@ -71,12 +78,6 @@
  * work for us too. */
 #define ADD_ENTROPY 32
 
-/* Use RAND_poll if openssl is 0.9.6 release or later.  (The "f" means
-   "release".)  */
-
-#define USE_RAND_POLL 0
-/* XXX Somehow setting USE_RAND_POLL on causes stack smashes. We're
- * not sure where. This was the big bug with Tor 0.1.1.9-alpha. */
 
 typedef QMap<QString, QString> servermap;
 servermap serverTofp_identity;
@@ -84,7 +85,11 @@ servermap fp_identityToServer;
 
 /** Intermediate information about the digest of a stream of data. */
 struct crypto_digest_env_t {
+#ifndef USE_OPENSSL
+  gcry_md_hd_t d;
+#else
   SHA_CTX d;
+#endif
 };
 
 
@@ -187,14 +192,7 @@ crypto_seed_rng(void)
   int fd;
   int i, n;
 
-#if USE_RAND_POLL
-  /* OpenSSL 0.9.6 adds a RAND_poll function that knows about more kinds of
-   * entropy than we do.  We'll try calling that, *and* calling our own entropy
-   * functions.  If one succeeds, we'll accept the RNG as seeded. */
-  rand_poll_status = RAND_poll();
-#else
   rand_poll_status = 0;
-#endif
 
   for (i = 0; filenames[i]; ++i) {
     fd = open(filenames[i], O_RDONLY, 0);
@@ -204,7 +202,11 @@ crypto_seed_rng(void)
     if (n != sizeof(buf)) {
       return -1;
     }
+#ifndef USE_OPENSSL
+    gcry_create_nonce(buf, sizeof(buf));
+#else
     RAND_seed(buf, sizeof(buf));
+#endif
     return 0;
   }
 
@@ -217,10 +219,15 @@ crypto_seed_rng(void)
 int
 crypto_rand(char *to, size_t n)
 {
-  int r;
   assert(to);
+#ifndef USE_OPENSSL
+  gcry_randomize((unsigned char*)to,n,GCRY_STRONG_RANDOM);
+  return 0;
+#else
+  int r;
   r = RAND_bytes((unsigned char*)to, n);
   return (r == 1) ? 0 : -1;
+#endif
 }
 
 /** Return a pseudorandom integer, chosen uniformly from the values
@@ -290,15 +297,24 @@ read_all(int fd, char *buf, size_t count, int isSocket)
 int
 digest_from_base64(char *digest, const char *d64)
 {
+
   char buf_in[BASE64_DIGEST_LEN+3];
   char buf[256];
   if (strlen(d64) != BASE64_DIGEST_LEN)
     return -1;
   memcpy(buf_in, d64, BASE64_DIGEST_LEN);
-  memcpy(buf_in+BASE64_DIGEST_LEN, "=\n\0", 3);
-
+#ifndef USE_OPENSSL
+  memcpy(buf_in+BASE64_DIGEST_LEN, "=\0", 2);
   if (base64_decode(buf, sizeof(buf), buf_in, strlen(buf_in)) != DIGEST_LEN)
     return -1;
+#else
+  memcpy(buf_in+BASE64_DIGEST_LEN, "=\n\0", 3);
+  if (base64_decode(buf, sizeof(buf), buf_in, strlen(buf_in)) != DIGEST_LEN)
+    return -1;
+
+#endif
+
+
   memcpy(digest, buf, DIGEST_LEN);
   return 0;
 }
@@ -307,8 +323,15 @@ int
 base64_decode(char *dest, size_t destlen, const char *src, size_t srclen)
 {
 
+#ifndef USE_OPENSSL
+//   gnutls_datum_t data_in;
+	base64_decodestate state;
+#else
   EVP_ENCODE_CTX ctx;
-  int len, ret;
+  int len;
+#endif
+  int ret;
+
   /* 64 bytes of input -> *up to* 48 bytes of output.
      Plus one more byte, in case I'm wrong.
   */
@@ -317,12 +340,29 @@ base64_decode(char *dest, size_t destlen, const char *src, size_t srclen)
   if (destlen > SIZE_T_CEILING)
     return -1;
 
+#ifndef USE_OPENSSL
+/*  data_in.data = (unsigned char*)src;
+  data_in.size = srclen;*/
+	base64_init_decodestate(&state);
+#endif
+
+#ifndef USE_OPENSSL
+//   if (gnutls_srp_base64_decode(&data_in, dest, &destlen) 
+//       == GNUTLS_E_SHORT_MEMORY_BUFFER)
+//       kdDebug() << "error decoding " << endl;
+//   kdDebug() << "decoded " << dest << "len" << destlen << endl;
+
+  ret = base64_decode_block(src, srclen, dest, &state);
+  return ret;
+#else
   EVP_DecodeInit(&ctx);
   EVP_DecodeUpdate(&ctx, (unsigned char*)dest, &len,
                    (unsigned char*)src, srclen);
   EVP_DecodeFinal(&ctx, (unsigned char*)dest, &ret);
   ret += len;
   return ret;
+#endif
+
 }
 
 int
@@ -338,8 +378,15 @@ digest_to_base64(char *d64, const char *digest)
 int
 base64_encode(char *dest, size_t destlen, const char *src, size_t srclen)
 {
+#ifndef USE_OPENSSL
+//   gnutls_datum_t data_in;
+  base64_encodestate state;
+#else
   EVP_ENCODE_CTX ctx;
-  int len, ret;
+  int len;
+#endif
+  int ret;
+
 
   /* 48 bytes of input -> 64 bytes of output plus newline.
      Plus one more byte, in case I'm wrong.
@@ -349,12 +396,29 @@ base64_encode(char *dest, size_t destlen, const char *src, size_t srclen)
   if (destlen > SIZE_T_CEILING)
     return -1;
 
+#ifndef USE_OPENSSL
+/*  data_in.data = (unsigned char*)src;
+  data_in.size = srclen;*/
+  base64_init_encodestate(&state);
+#endif
+
+#ifndef USE_OPENSSL
+//   gnutls_srp_base64_encode(&data_in, dest, &destlen);
+//   kdDebug() << "encoded " << dest << "len" << destlen << endl;
+//   return destlen;
+  ret = base64_encode_block(src, srclen, dest, &state);
+  ret += base64_encode_blockend(dest+ret, &state);
+
+  return ret;
+#else
   EVP_EncodeInit(&ctx);
   EVP_EncodeUpdate(&ctx, (unsigned char*)dest, &len,
                    (unsigned char*)src, srclen);
   EVP_EncodeFinal(&ctx, (unsigned char*)(dest+len), &ret);
   ret += len;
   return ret;
+#endif
+
 }
 
 static const char HEX_DIGITS[] = "0123456789ABCDEFabcdef";
@@ -432,7 +496,12 @@ crypto_new_digest_env(void)
 {
   crypto_digest_env_t *r;
   r = (crypto_digest_env_t *)malloc(sizeof(crypto_digest_env_t));
+
+#ifndef USE_OPENSSL
+  gcry_md_open(&r->d,GCRY_MD_SHA1, 0);
+#else
   SHA1_Init(&r->d);
+#endif
   return r;
 }
 
@@ -444,15 +513,30 @@ void
 crypto_digest_get_digest(crypto_digest_env_t *digest,
                          char *out, size_t out_len)
 {
+
+#ifdef USE_OPENSSL
   static unsigned char r[DIGEST_LEN];
-  SHA_CTX tmpctx;
+#else
+  unsigned char* r;
+#endif
   assert(digest);
   assert(out);
   assert(out_len <= DIGEST_LEN);
+#ifdef USE_OPENSSL
+  SHA_CTX tmpctx;
+#endif
+
+#ifndef USE_OPENSSL
+  r = gcry_md_read(digest->d, GCRY_MD_SHA1);
+  memcpy(out, r, out_len);
+  gcry_md_close(digest->d);
+#else
   /* memcpy into a temporary ctx, since SHA1_Final clears the context */
   memcpy(&tmpctx, &digest->d, sizeof(SHA_CTX));
   SHA1_Final(r, &tmpctx);
   memcpy(out, r, out_len);
+#endif
+
 }
 
 /** Add <b>len</b> bytes from <b>data</b> to the digest object.
@@ -468,7 +552,12 @@ crypto_digest_add_bytes(crypto_digest_env_t *digest, const char *data,
    * to the hardware, and hearing the answer, is likely higher than
    * just doing it ourselves. Hashes are fast.
    */
+#ifndef USE_OPENSSL
+  gcry_md_write(digest->d, data, len);
+#else
   SHA1_Update(&digest->d, (void*)data, len);
+#endif
+
 }
 
 /** Implement RFC2440-style iterated-salted S2K conversion: convert the
@@ -528,6 +617,200 @@ QString hashPassword(const char* secret)
                 secret, strlen(secret),
                 key);
   base16_encode(output, sizeof(output), key, sizeof(key));
+  kdDebug() << output << endl;
   return output;
 }
 
+/*
+cdecoder.c - c source to a base64 decoding algorithm implementation
+
+This is part of the libb64 project, and has been placed in the public domain.
+For details, see http://sourceforge.net/projects/libb64
+*/
+
+int base64_decode_value(char value_in)
+{
+	static const char decoding[] = {62,-1,-1,-1,63,52,53,54,55,56,57,58,59,60,61,-1,-1,-1,-2,-1,-1,-1,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,-1,-1,-1,-1,-1,-1,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51};
+	static const char decoding_size = sizeof(decoding);
+	value_in -= 43;
+	if (value_in < 0 || value_in > decoding_size) return -1;
+	return decoding[(int)value_in];
+}
+
+void base64_init_decodestate(base64_decodestate* state_in)
+{
+	state_in->step = step_a;
+	state_in->plainchar = 0;
+}
+
+int base64_decode_block(const char* code_in, const int length_in, char* plaintext_out, base64_decodestate* state_in)
+{
+	const char* codechar = code_in;
+	char* plainchar = plaintext_out;
+	char fragment;
+	
+	*plainchar = state_in->plainchar;
+	
+	switch (state_in->step)
+	{
+		while (1)
+		{
+	case step_a:
+			do {
+				if (codechar == code_in+length_in)
+				{
+					state_in->step = step_a;
+					state_in->plainchar = *plainchar;
+					return plainchar - plaintext_out;
+				}
+				fragment = (char)base64_decode_value(*codechar++);
+			} while (fragment < 0);
+			*plainchar    = (fragment & 0x03f) << 2;
+	case step_b:
+			do {
+				if (codechar == code_in+length_in)
+				{
+					state_in->step = step_b;
+					state_in->plainchar = *plainchar;
+					return plainchar - plaintext_out;
+				}
+				fragment = (char)base64_decode_value(*codechar++);
+			} while (fragment < 0);
+			*plainchar++ |= (fragment & 0x030) >> 4;
+			*plainchar    = (fragment & 0x00f) << 4;
+	case step_c:
+			do {
+				if (codechar == code_in+length_in)
+				{
+					state_in->step = step_c;
+					state_in->plainchar = *plainchar;
+					return plainchar - plaintext_out;
+				}
+				fragment = (char)base64_decode_value(*codechar++);
+			} while (fragment < 0);
+			*plainchar++ |= (fragment & 0x03c) >> 2;
+			*plainchar    = (fragment & 0x003) << 6;
+	case step_d:
+			do {
+				if (codechar == code_in+length_in)
+				{
+					state_in->step = step_d;
+					state_in->plainchar = *plainchar;
+					return plainchar - plaintext_out;
+				}
+				fragment = (char)base64_decode_value(*codechar++);
+			} while (fragment < 0);
+			*plainchar++   |= (fragment & 0x03f);
+		}
+	}
+	/* control should not reach here */
+	return plainchar - plaintext_out;
+}
+
+/*
+cencoder.c - c source to a base64 encoding algorithm implementation
+
+This is part of the libb64 project, and has been placed in the public domain.
+For details, see http://sourceforge.net/projects/libb64
+*/
+
+
+const int CHARS_PER_LINE = 72;
+
+void base64_init_encodestate(base64_encodestate* state_in)
+{
+	state_in->step = step_A;
+	state_in->result = 0;
+	state_in->stepcount = 0;
+}
+
+char base64_encode_value(char value_in)
+{
+	static const char* encoding = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+	if (value_in > 63) return '=';
+	return encoding[(int)value_in];
+}
+
+int base64_encode_block(const char* plaintext_in, int length_in, char* code_out, base64_encodestate* state_in)
+{
+	const char* plainchar = plaintext_in;
+	const char* const plaintextend = plaintext_in + length_in;
+	char* codechar = code_out;
+	char result;
+	char fragment;
+	
+	result = state_in->result;
+	
+	switch (state_in->step)
+	{
+		while (1)
+		{
+	case step_A:
+			if (plainchar == plaintextend)
+			{
+				state_in->result = result;
+				state_in->step = step_A;
+				return codechar - code_out;
+			}
+			fragment = *plainchar++;
+			result = (fragment & 0x0fc) >> 2;
+			*codechar++ = base64_encode_value(result);
+			result = (fragment & 0x003) << 4;
+	case step_B:
+			if (plainchar == plaintextend)
+			{
+				state_in->result = result;
+				state_in->step = step_B;
+				return codechar - code_out;
+			}
+			fragment = *plainchar++;
+			result |= (fragment & 0x0f0) >> 4;
+			*codechar++ = base64_encode_value(result);
+			result = (fragment & 0x00f) << 2;
+	case step_C:
+			if (plainchar == plaintextend)
+			{
+				state_in->result = result;
+				state_in->step = step_C;
+				return codechar - code_out;
+			}
+			fragment = *plainchar++;
+			result |= (fragment & 0x0c0) >> 6;
+			*codechar++ = base64_encode_value(result);
+			result  = (fragment & 0x03f) >> 0;
+			*codechar++ = base64_encode_value(result);
+			
+			++(state_in->stepcount);
+			if (state_in->stepcount == CHARS_PER_LINE/4)
+			{
+				*codechar++ = '\n';
+				state_in->stepcount = 0;
+			}
+		}
+	}
+	/* control should not reach here */
+	return codechar - code_out;
+}
+
+int base64_encode_blockend(char* code_out, base64_encodestate* state_in)
+{
+	char* codechar = code_out;
+	
+	switch (state_in->step)
+	{
+	case step_B:
+		*codechar++ = base64_encode_value(state_in->result);
+/*		*codechar++ = '=';
+		*codechar++ = '=';*/
+		break;
+	case step_C:
+		*codechar++ = base64_encode_value(state_in->result);
+/*		*codechar++ = '=';*/
+		break;
+	case step_A:
+		break;
+	}
+/*	*codechar++ = '\n';*/
+	
+	return codechar - code_out;
+}
