@@ -22,6 +22,7 @@
 #include "functions.h"
 #include <qtimer.h>
 #include <kmessagebox.h>
+#include <cstdlib>
 
 bool local;
 QString thttpd;
@@ -71,7 +72,7 @@ void HidSrvWizard::localWeb_toggled( bool state)
 
 void HidSrvWizard::downloadThttpd_clicked()
 {
-    TorkUpdate* updater = new TorkUpdate(this,false);
+    TorkUpdate* updater = new TorkUpdate(this);
     updater->downloadThttpd();
 
 }
