@@ -1,5 +1,5 @@
 /***************************************************************************
- ** $Id: newfirstrunwizard.ui.h,v 1.21 2007/12/21 19:14:26 hoganrobert Exp $
+ ** $Id: newfirstrunwizard.ui.h,v 1.25 2008/06/02 10:38:26 hoganrobert Exp $
  *   Copyright (C) 2006 by Robert Hogan                                    *
  *   robert@roberthogan.net                                                *
  *                                                                         *
@@ -40,16 +40,16 @@
 #include <krun.h>
 #include <qvaluevector.h>
 
-#include "libwhich/which.h"
+#include "functions.h"
 
 TorClient* client;
 KProcIO *whichproc;
 bool torRunning;
 bool privoxyRunning;
-char *tor = NULL;
-char *privoxy = NULL;
-char *netstat = NULL;
-char *torknetstat = NULL;
+QString tor;
+QString privoxy;
+QString netstat;
+QString torknetstat;
 
 QString OriginalHttpProxy;
 QString OriginalHttpsProxy;
@@ -106,18 +106,11 @@ FirstRunWizard::init()
     setAppropriate ( TorServer, false );
 
     TorkConfig::setUser(getenv("USER"));
+    const char *paths = ":/usr/local/sbin:/usr/sbin:/sbin:/usr/local/bin:/usr/bin:/bin";
 
-    QString paths = (QString) getenv("PATH");
-    paths += ":/usr/local/sbin:/usr/sbin:/sbin:/usr/local/bin:/usr/bin:/bin";
-    paths.replace("::",":");
-    const char *path_list = paths;
-    int path_index = 0;
-
-    privoxy = find_command_in_path("privoxy", path_list, &path_index);
-    path_index = 0;
-    tor = find_command_in_path("tor", path_list, &path_index);
-    path_index = 0;
-    netstat = find_command_in_path("netstat", path_list, &path_index);
+    privoxy = getFullLocation(paths,"privoxy");
+    tor = getFullLocation(paths,"tor");
+    netstat = getFullLocation(paths,"netstat");
 
 
     if (!tor)
@@ -186,6 +179,7 @@ void FirstRunWizard::usingAnotherProxy_toggled( bool state)
     if ((!privoxy) && (!state))
         setAppropriate ( Locate_Privoxy, !state );
     setAppropriate ( HowDoesPrivoxyStart, !state );
+    setAppropriate ( PrivoxyConfiguration, !state );
     setAppropriate ( FinalPrivoxy, !state );
 }
 
@@ -277,7 +271,7 @@ FirstRunWizard::showPage( QWidget *w ) //virtual
 
 void FirstRunWizard::downloadPrivoxy_clicked()
 {
-    TorkUpdate* updater = new TorkUpdate(this,true);
+    TorkUpdate* updater = new TorkUpdate(this);
     updater->checkForNewPrivoxyDirectly();
 
 }
@@ -286,7 +280,7 @@ void FirstRunWizard::downloadPrivoxy_clicked()
 void FirstRunWizard::downloadTor_clicked()
 {
 
-    TorkUpdate* updater = new TorkUpdate(this,true);
+    TorkUpdate* updater = new TorkUpdate(this);
     updater->checkForNewTorDirectly();
 
 }
@@ -295,13 +289,11 @@ void
 FirstRunWizard::accept()
 {
 
-    QString paths = (QString) getenv("PATH");
-    paths += ":/usr/local/sbin:/usr/sbin:/sbin:/usr/local/bin:/usr/bin:/bin";
+    QString paths;
+    paths = ":/usr/local/sbin:/usr/sbin:/sbin:/usr/local/bin:/usr/bin:/bin";
     paths.replace("::",":");
-    const char *path_list = paths;
 
-    int path_index = 0;
-    torknetstat = find_command_in_path("torknetstat", path_list, &path_index);
+    torknetstat = getFullLocation(paths,"torknetstat");;
 
     if (torknetstat)
         TorkConfig::setNetstatLocation("torknetstat");
@@ -593,8 +585,6 @@ void FirstRunWizard::examineConfigFiles()
             "When you've entered your password "
             "click 'Test Tor' to see if it worked. "));
     }
-
-
 }
 
 void FirstRunWizard::pushButton6_clicked()
@@ -615,12 +605,12 @@ void FirstRunWizard::rootifyNetstat()
 
 }
 
-void FirstRunWizard::sighupTor()
+void FirstRunWizard::sighupTor(const QString& text)
 {
 
     KProcIO *catproc = new KProcIO();
     catproc->setUseShell(TRUE);
-    QString whichCommand= "killall -s HUP tor";
+    QString whichCommand= QString("killall -s HUP %1").arg(text);
     *catproc<<whichCommand;
     catproc->start(KProcIO::NotifyOnExit,TRUE);
 
@@ -648,7 +638,7 @@ void FirstRunWizard::appendControlDirective(const QString& text)
 //         ts << "CookieAuthentication 1" << "\n";
 
         inf.close();
-        sighupTor();
+        sighupTor("tor");
 
     }else{
 
@@ -687,3 +677,92 @@ void FirstRunWizard::RemoteTorAddress_textChanged( const QString & text)
     else
         TorPassword->setEnabled(true);
 }
+
+
+void FirstRunWizard::modifyConfPrivoxy_clicked()
+{
+  examinePrivoxyConfigFiles();
+}
+
+void FirstRunWizard::examinePrivoxyConfigFiles()
+{
+
+    bool foundone = false;
+
+    for (unsigned int index = 0; index != configCandidatesPrivoxy->count(); ++index){
+		if (QFile::exists(configCandidatesPrivoxy->item(index)->text()))
+		{
+			appendPrivoxyConfig(configCandidatesPrivoxy->item(index)->text());
+            foundone = true;
+		}
+    }
+
+
+    if (!foundone){
+        appendPrivoxyConfig("/etc/privoxy/config");
+        privoxyStatus->setText(i18n("<p>OK, I didn't find any of the config files in the list.<br>"
+            "Tork has created a basic config in /etc/privoxy/config."
+            "This <i>may</i> get things working, but possibly not.<br>"
+            "You should:"
+            "- Check Privoxy is properly installed."
+            "- Re-install privoxy and try running the wizard again."
+             ));
+    }
+}
+
+void FirstRunWizard::appendPrivoxyConfig(const QString& text)
+{
+
+    QString privoxyConf = QString(
+            "confdir %1\\n"
+            "logdir .\\n"
+            "listen-address %2:%3\\n"
+            "debug   1    # URLs\\n"
+            "debug   4096 # Info\\n"
+            "debug   8192 # Errors\\n"
+            "toggle 1\\n"
+            "buffer-limit 4069\\n"
+            "forward         192.168.*.*/     .\\n"
+            "forward            10.*.*.*/     .\\n"
+            "forward           127.*.*.*/     .\\n"
+            "forward-socks4a / %4:%5 .\\n")
+            .arg(locate("data","tork/privoxy/"))
+            .arg(TorkConfig::konqHttpProxy().replace("http://",""))
+            .arg(TorkConfig::konqHttpProxyPort())
+            .arg(TorkConfig::sOCKSBindAddressHost())
+            .arg(TorkConfig::sOCKSBindAddressPort());
+
+    int result = KMessageBox::questionYesNo(this, 
+                 i18n( "<p>I'm going to modify the Privoxy configuration file: <b>%1</b>.<br>"
+                  "This is so that I can ensure"
+                  "Privoxy can communicate with Tor.<br>"
+                  " If you say Yes, I'll ask for your root password.").arg(text));
+    switch (result) {
+        case KMessageBox::No : 
+            return;
+    }
+
+    QFile inf(text);
+    if ( inf.open(IO_WriteOnly | IO_Append) ) {
+
+        QTextStream ts( &inf );
+
+        ts << privoxyConf;
+
+        inf.close();
+        sighupTor("privoxy");
+
+    }else{
+
+        KProcIO *catproc = new KProcIO();
+        catproc->setUseShell(TRUE);
+        QString whichCommand= QString("kdesu -c 'printf \"%1\""
+                                      " >> %2;killall -s HUP privoxy'")
+                                      .arg(privoxyConf)
+                                      .arg(text);
+        *catproc<<whichCommand;
+        catproc->start(KProcIO::NotifyOnExit,TRUE);
+
+    }
+
+}
