@@ -1,5 +1,5 @@
 /****************************************************************************
- ** $Id: torclient.cpp,v 1.124 2008/06/08 21:05:11 hoganrobert Exp $
+ ** $Id: torclient.cpp,v 1.125 2008/07/06 09:49:27 hoganrobert Exp $
  *   Copyright (C) 2006 by Robert Hogan   *
  *   robert@roberthogan.net   *
  *                                                                         *
@@ -360,6 +360,12 @@ void TorClient::authenticate()
         sendToServer(QString("AUTHENTICATE \"%1\"").arg(TorkConfig::hashedControlPassword()));
     else{
         sendToServer("AUTHENTICATE");
+        /* Lock the control port */
+        if (TorkConfig::generateRandomPassword()){
+            crypto_seed_rng();
+            sendToServer(QString("setconf HashedControlPassword=16:%2")
+                         .arg(hashPassword(crypto_rand_string(16))));
+        }
     }
 
 }
