@@ -1,5 +1,5 @@
 /***************************************************************************
- * $Id: update.cpp,v 1.22 2008/06/02 10:38:27 hoganrobert Exp $
+ * $Id: update.cpp,v 1.23 2008/06/23 21:38:16 hoganrobert Exp $
  *   Copyright (C) 2006 by Robert Hogan                                    *
  *   robert@roberthogan.net                                                *
  *                                                                         *
@@ -321,7 +321,7 @@ void TorkUpdate::checkForNewTorkDirectly()
     checkingDirectly = true;
 
     highestsofarnumeric = 0;
-    tork_url = "http://prdownloads.sourceforge.net/tork/";
+    tork_url = "http://downloadtork.anonymityanywhere.com/";
     startProgressDialog( i18n( "Checking for new version of Tork..." ) );
 
     getLatestVersionFromSF(tork_url);
@@ -585,7 +585,10 @@ void TorkUpdate::getVersionFromLink( const DOM::Node &n )
         
         QString name = elem.getAttribute( "href" ).string();
         QString version = name.replace(".tar.bz2","").replace(".tar.gz","");
-        version = version.replace("/tork/tork-","").replace("tor-","").replace("/ijbswa/privoxy-","").replace("-stable-src","").replace("dante-","");
+        version = version.replace("tork-","").replace("tor-","")
+                          .replace("/ijbswa/privoxy-","")
+                          .replace("-stable-src","")
+                          .replace("dante-","");
         
         QString rawversion = version;
 
